/*
 * Part of Intel(R) Manageability Engine Interface Linux driver
 *
 * Copyright (c) 2003 - 2008 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 */

#include "heci.h"


/**
 * heci_set_csr_register - write H_CSR register to the heci device
 *
 * @dev: device object for our driver
 *
 * NOTE: called under hw_lock
 */
void heci_set_csr_register(struct iamt_heci_device *dev)
{
	write_heci_register(dev, H_CSR, dev->host_hw_state);
	dev->host_hw_state = read_heci_register(dev, H_CSR);
}

/**
 * heci_csr_enable_interrupts - enable heci device interrupts
 *
 * @dev: device object for our driver
 *
 * NOTE: called under hw_lock
 */
void heci_csr_enable_interrupts(struct iamt_heci_device *dev)
{
	dev->host_hw_state |= H_IE;
	heci_set_csr_register(dev);
}

/**
 * heci_csr_disable_interrupts - disable heci device interrupts
 *
 * @dev: device object for our driver
 *
 * NOTE: called under hw_lock
 */
void heci_csr_disable_interrupts(struct iamt_heci_device *dev)
{
	dev->host_hw_state &= ~H_IE;
	heci_set_csr_register(dev);
}

/**
 * heci_csr_gen_interrupt - interrupt ME
 *
 * @dev: device object for our driver
 *
 * NOTE: called under hw_lock
 */
static void heci_csr_gen_interrupt(struct iamt_heci_device *dev)
{
	dev->host_hw_state |= H_IG;
	dev->host_hw_state &= ~H_IS;
	heci_set_csr_register(dev);
}

/**
 * _host_get_filled_slots - get number of device filled buffer slots
 *
 * @device: the device structure
 *
 * returns numer of filled slots
 *
 * NOTE: called under hw_lock
 */
static unsigned char _host_get_filled_slots(const struct iamt_heci_device *dev)
{
	char read_ptr, write_ptr;

	read_ptr = (char) ((dev->host_hw_state & H_CBRP) >> 8);
	write_ptr = (char) ((dev->host_hw_state & H_CBWP) >> 16);

	return (unsigned char) (write_ptr - read_ptr);
}

/**
 * host_buffer_is_empty  - check if host buffer is empty.
 *
 * @dev: device object for our driver
 * @eslots: (out) -1(ESLOTS_OVERFLOW) if overflow,
 *               otherwise empty slots count
 *
 * returns  1 if empty, 0 - otherwise.
 */
int host_buffer_is_empty(struct iamt_heci_device *dev, __s32 *eslots)
{
	unsigned char buffer_depth, filled_slots, empty_slots;
	int ret;

	spin_lock(&dev->hw_lock);
	dev->host_hw_state = read_heci_register(dev, H_CSR);
	buffer_depth = (unsigned char) ((dev->host_hw_state & H_CBD) >> 24);
	filled_slots = _host_get_filled_slots(dev);
	spin_unlock(&dev->hw_lock);
	ret = (filled_slots > 0) ? 0 : 1;
	empty_slots = buffer_depth - filled_slots;

	if (eslots != NULL) {
		if (filled_slots > buffer_depth)
			*eslots = -ESLOTS_OVERFLOW;
		else
			*eslots = (__s32) empty_slots;
		atomic_set(&dev->host_buffer_is_empty, 0);
	} else
		atomic_set(&dev->host_buffer_is_empty, ret);

	return ret;
}

/**
 * heci_write_message  - write a message to heci device.
 *
 * @dev: device object for our driver
 * @heci_hdr: header of  message
 * @write_buffer: message buffer will be write
 * @write_length: message size will be write
 *
 * returns 1 if success, 0 - otherwise.
 */
int heci_write_message(struct iamt_heci_device *dev,
			     struct heci_msg_hdr *header,
			     unsigned char *write_buffer,
			     unsigned long write_length)
{
	__u32 temp_msg = 0;
	unsigned long bytes_written = 0;
	unsigned char buffer_depth, filled_slots, empty_slots;
	unsigned long dw_to_write;

	spin_lock(&dev->hw_lock);
	dev->host_hw_state = read_heci_register(dev, H_CSR);
	DBG("host_hw_state = 0x%08x.\n", dev->host_hw_state);
	DBG("heci_write_message header=%08x.\n", *((__u32 *) header));
	buffer_depth = (unsigned char) ((dev->host_hw_state & H_CBD) >> 24);
	filled_slots = _host_get_filled_slots(dev);
	empty_slots = buffer_depth - filled_slots;
	DBG("filled = %hu, empty = %hu.\n", filled_slots, empty_slots);

	dw_to_write = ((write_length + 3) / 4);

	if (dw_to_write > empty_slots) {
		spin_unlock(&dev->hw_lock);
		return 0;
	}

	write_heci_register(dev, H_CB_WW, *((__u32 *) header));

	while (write_length >= 4) {
		write_heci_register(dev, H_CB_WW,
				*(__u32 *) (write_buffer + bytes_written));
		bytes_written += 4;
		write_length -= 4;
	}

	if (write_length > 0) {
		memcpy(&temp_msg, &write_buffer[bytes_written], write_length);
		write_heci_register(dev, H_CB_WW, temp_msg);
	}

	heci_csr_gen_interrupt(dev);

	dev->me_hw_state = read_heci_register(dev, ME_CSR_HA);
	if ((dev->me_hw_state & ME_RDY_HRA) != ME_RDY_HRA) {
		spin_unlock(&dev->hw_lock);
		return 0;
	}

	spin_unlock(&dev->hw_lock);
	return 1;
}

/**
 * count_full_read_slots  - count read full slots.
 *
 * @dev: device object for our driver
 *
 * returns -1(ESLOTS_OVERFLOW) if overflow, otherwise filled slots count
 */
__s32 count_full_read_slots(struct iamt_heci_device *dev)
{
	char read_ptr, write_ptr;
	unsigned char buffer_depth, filled_slots;

	spin_lock(&dev->hw_lock);
	dev->me_hw_state = read_heci_register(dev, ME_CSR_HA);
	buffer_depth = (unsigned char) ((dev->me_hw_state & ME_CBD_HRA) >> 24);
	read_ptr = (char) ((dev->me_hw_state & ME_CBRP_HRA) >> 8);
	write_ptr = (char) ((dev->me_hw_state & ME_CBWP_HRA) >> 16);
	spin_unlock(&dev->hw_lock);

	filled_slots = (unsigned char) (write_ptr - read_ptr);

	if (filled_slots > buffer_depth) {
		/* overflow */
		return -ESLOTS_OVERFLOW;
	}

	DBG("filled_slots =%08x  \n", filled_slots);
	return (__s32) filled_slots;
}

/**
 * heci_read_slots  - read a message from heci device.
 *
 * @dev: device object for our driver
 * @buffer: message buffer will be write
 * @buffer_length: message size will be read
 *
 * NOTE: read from ME_CB_RW register is not locked
 *     - called only from heci_bh_read_handler subroutines
 */
void heci_read_slots(struct iamt_heci_device *dev,
		     unsigned char *buffer, unsigned long buffer_length)
{
	__u32 i = 0;
	unsigned char temp_buf[sizeof(__u32)];

	while (buffer_length >= sizeof(__u32)) {
		((__u32 *) buffer)[i] = read_heci_register(dev, ME_CB_RW);
		DBG("buffer[%d]= 0x%x\n", i, ((__u32 *) buffer)[i]);
		i++;
		buffer_length -= sizeof(__u32);
	}

	if (buffer_length > 0) {
		*((__u32 *) &temp_buf) = read_heci_register(dev, ME_CB_RW);
		memcpy(&buffer[i * 4], temp_buf, buffer_length);
	}

	spin_lock(&dev->hw_lock);
	heci_csr_gen_interrupt(dev);
	spin_unlock(&dev->hw_lock);
}

/**
 * flow_ctrl_creds  - check flow_control credentials.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 *
 * returns 1 if flow_ctrl_creds >0, 0 - otherwise.
 */
int flow_ctrl_creds(struct iamt_heci_device *dev,
				   struct heci_connection *conn)
{
	__u8 i;

	spin_lock(&dev->mecl_lock);

	if (dev->num_heci_me_clients == 0) {
		spin_unlock(&dev->mecl_lock);
		return 0;
	}

	if (conn == NULL) {
		spin_unlock(&dev->mecl_lock);
		return 0;
	}

	if (conn->flow_ctrl_creds > 0) {
		spin_unlock(&dev->mecl_lock);
		return 1;
	}

	for (i = 0; i < dev->num_heci_me_clients; i++) {
		if (dev->me_clients[i].client_id == conn->me_client_id) {
			if (dev->me_clients[i].flow_ctrl_creds > 0) {
				BUG_ON(dev->me_clients[i].props.single_recv_buf
					 == 0);
				spin_unlock(&dev->mecl_lock);
				return 1;
			}
			spin_unlock(&dev->mecl_lock);
			return 0;
		}
	}
	BUG();

	spin_unlock(&dev->mecl_lock);
	return 0;
}

/**
 * flow_ctrl_reduce  - reduce flow_control.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 */
void flow_ctrl_reduce(struct iamt_heci_device *dev,
			 struct heci_connection *conn)
{
	__u8 i;

	spin_lock(&dev->mecl_lock);
	if (dev->num_heci_me_clients == 0) {
		spin_unlock(&dev->mecl_lock);
		return;
	}

	for (i = 0; i < dev->num_heci_me_clients; i++) {
		if (dev->me_clients[i].client_id == conn->me_client_id) {
			if (dev->me_clients[i].props.single_recv_buf != 0) {
				BUG_ON(dev->me_clients[i].flow_ctrl_creds <= 0);
				dev->me_clients[i].flow_ctrl_creds--;
			} else {
				BUG_ON(conn->flow_ctrl_creds <= 0);
				conn->flow_ctrl_creds--;
			}
			spin_unlock(&dev->mecl_lock);
			return;
		}
	}
	BUG();
	spin_unlock(&dev->mecl_lock);
}

/**
 * heci_client_set_props  - set heci client properties.
 *
 * @dev: device object for our driver
 * @props_res: properties response from heci
 */
void heci_client_set_props(struct iamt_heci_device *dev,
			    struct hbm_props_response *props_res)
{
	__u8 i;

	if (props_res->status != 0) {
		BUG();
		return;
	}

	spin_lock(&dev->mecl_lock);
	if (dev->num_heci_me_clients == 0) {
		spin_unlock(&dev->mecl_lock);
		return;
	}

	for (i = 0; i < dev->num_heci_me_clients; i++) {
		if (dev->me_clients[i].client_id == props_res->address) {
			dev->me_clients[i].props = props_res->client_properties;
			break;
		}
	}
	spin_unlock(&dev->mecl_lock);
}


/**
 * heci_client_find  - find heci client.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 * @idx: (out) pointer to client index
 *
 * returns -ENODEV if not found, 0 if found
 */
int heci_client_find(struct iamt_heci_device *dev,
			 struct heci_connection *conn, __u8 *idx)
{
	__u8 i;

	if (dev->num_heci_me_clients == 0)
		return -ENODEV;

	for (i = 0; i < dev->num_heci_me_clients; i++) {
		if (dev->me_clients[i].client_id == conn->me_client_id)
			break;
	}
	if ((i == dev->num_heci_me_clients)
	    || (dev->me_clients[i].client_id != conn->me_client_id))
		return -ENODEV;

	*idx = i;
	return 0;
}


/**
 * heci_send_flow_control - send flow control to fw.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 *
 * returns 1 if success, 0 - otherwise.
 */
int heci_send_flow_control(struct iamt_heci_device *dev,
				 struct heci_connection *conn)
{
	struct heci_msg_hdr *heci_hdr;
	struct hbm_flow_control *heci_flow_control;
	int ret = 0;

	spin_lock(&dev->write_lock);
	heci_hdr = (struct heci_msg_hdr *) &dev->wr_msg_buf[0];
	heci_hdr->host_addr = 0;
	heci_hdr->me_addr = 0;
	heci_hdr->length = sizeof(struct hbm_flow_control);
	heci_hdr->msg_complete = 1;
	heci_hdr->reserved = 0;

	heci_flow_control = (struct hbm_flow_control *) &dev->wr_msg_buf[1];
	memset(heci_flow_control, 0, sizeof(struct hbm_flow_control));
	heci_flow_control->host_addr = conn->host_client_id;
	heci_flow_control->me_addr = conn->me_client_id;
	heci_flow_control->cmd.cmd = HECI_FLOW_CONTROL_CMD;
	DBG("sending flow control host client = %d, me client = %d\n",
	    conn->host_client_id, conn->me_client_id);
	if (heci_write_message(dev, heci_hdr,
				(unsigned char *) heci_flow_control,
				sizeof(struct hbm_flow_control)))
		ret = 1;

	spin_unlock(&dev->write_lock);
	return ret;
}

/**
 * other_client_is_connecting  - check if other
 *   client with the same client id is connected.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 *
 * returns 1 if other client is connected, 0 - otherwise.
 */
int other_client_is_connecting(struct iamt_heci_device *dev,
		struct heci_connection *conn)
{
	struct heci_connection *conn_pos = NULL;
	struct heci_connection *conn_next = NULL;

	spin_lock(&dev->conn_lock);
	list_for_each_entry_safe(conn_pos, conn_next, &dev->conn_list, link) {
		if ((atomic_read(&conn_pos->state) == HECI_FILE_CONNECTING)
			&& (conn_pos != conn)
			&& conn->me_client_id == conn_pos->me_client_id) {
			spin_unlock(&dev->conn_lock);
			return 1;
		}
	}
	spin_unlock(&dev->conn_lock);

	return 0;
}

/**
 * heci_send_wd  - send watch dog message to fw.
 *
 * @dev: device object for our driver
 *
 * returns 1 if success, 0 - otherwise.
 */
int heci_send_wd(struct iamt_heci_device *dev)
{
	struct heci_msg_hdr *heci_hdr;
	int ret = 0;

	spin_lock(&dev->write_lock);
	heci_hdr = (struct heci_msg_hdr *) &dev->wr_msg_buf[0];
	heci_hdr->host_addr = dev->wd_conn.host_client_id;
	heci_hdr->me_addr = dev->wd_conn.me_client_id;
	heci_hdr->msg_complete = 1;
	heci_hdr->reserved = 0;

	if (!memcmp(dev->wd_data, heci_start_wd_params, HECI_WD_PARAMS_SIZE))
		heci_hdr->length = HECI_START_WD_DATA_SIZE;
	else {
		BUG_ON(memcmp(dev->wd_data, heci_stop_wd_params,
			HECI_WD_PARAMS_SIZE));
		heci_hdr->length = HECI_WD_PARAMS_SIZE;
	}

	if (heci_write_message(dev, heci_hdr, dev->wd_data, heci_hdr->length))
		ret = 1;

	spin_unlock(&dev->write_lock);
	return ret;
}

/**
 * heci_disconnect  - send disconnect message to fw.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 *
 * returns 1 if success, 0 - otherwise.
 */
int heci_disconnect(struct iamt_heci_device *dev,
			  struct heci_connection *conn)
{
	struct heci_msg_hdr *heci_hdr;
	struct hbm_client_disconnect_request *heci_cli_disconnect;
	int ret = 0;

	spin_lock(&dev->write_lock);
	heci_hdr = (struct heci_msg_hdr *) &dev->wr_msg_buf[0];
	heci_hdr->host_addr = 0;
	heci_hdr->me_addr = 0;
	heci_hdr->length = sizeof(struct hbm_client_disconnect_request);
	heci_hdr->msg_complete = 1;
	heci_hdr->reserved = 0;

	heci_cli_disconnect =
	    (struct hbm_client_disconnect_request *) &dev->wr_msg_buf[1];
	memset(heci_cli_disconnect, 0,
		sizeof(struct hbm_client_disconnect_request));
	heci_cli_disconnect->host_addr = conn->host_client_id;
	heci_cli_disconnect->me_addr = conn->me_client_id;
	heci_cli_disconnect->cmd.cmd = CLIENT_DISCONNECT_REQ_CMD;

	if (heci_write_message(dev, heci_hdr,
				(unsigned char *) heci_cli_disconnect,
				sizeof(struct hbm_client_disconnect_request)))
		ret = 1;

	spin_unlock(&dev->write_lock);
	return ret;
}

/**
 * heci_connect - send connect message to fw.
 *
 * @dev: device object for our driver
 * @conn: connection data of the file object
 *
 * returns 1 if success, 0 - otherwise.
 */
int heci_connect(struct iamt_heci_device *dev,
		       struct heci_connection *conn)
{
	struct heci_msg_hdr *heci_hdr;
	struct hbm_client_connect_request *heci_cli_connect;
	int ret = 0;

	DBG("connect pre.\n");
	spin_lock(&dev->write_lock);
	DBG("connect in.\n");
	heci_hdr = (struct heci_msg_hdr *) &dev->wr_msg_buf[0];
	heci_hdr->host_addr = 0;
	heci_hdr->me_addr = 0;
	heci_hdr->length = sizeof(struct hbm_client_connect_request);
	heci_hdr->msg_complete = 1;
	heci_hdr->reserved = 0;

	heci_cli_connect =
	    (struct hbm_client_connect_request *) &dev->wr_msg_buf[1];
	memset(heci_cli_connect, 0, sizeof(struct hbm_client_connect_request));
	heci_cli_connect->host_addr = conn->host_client_id;
	heci_cli_connect->me_addr = conn->me_client_id;
	heci_cli_connect->cmd.cmd = CLIENT_CONNECT_REQ_CMD;

	if (heci_write_message(dev, heci_hdr,
				(unsigned char *) heci_cli_connect,
				sizeof(struct hbm_client_connect_request)))
		ret = 1;

	spin_unlock(&dev->write_lock);
	DBG("connect post.\n");
	return ret;
}

/**
 * heci_flush_ctrl_lists - remove callback block from lists:
 *  heci_cmd_list, ctrl_wr_list, ctrl_rd_list
 *
 * @dev: Device object for our driver
 * @conn: connection data of the file object
 * @priv_cb: callback black to be removed
 *
 * returns 1 if removed, 0 if not found on lists.
 */
int heci_flush_ctrl_lists(struct iamt_heci_device *dev,
			struct heci_connection *conn,
			struct heci_cb_private *priv_cb)
{
	int ret;

	if ((dev == NULL) || (conn == NULL))
		return 0;

	if (priv_cb == NULL)
		return 0;

	spin_lock_bh(&dev->cmdl_lock);
	ret = heci_flush_list(&dev->heci_cmd_list, conn, priv_cb);
	spin_unlock_bh(&dev->cmdl_lock);
	if (ret != 0)
		return ret;

	spin_lock_bh(&dev->crrdl_lock);
	ret = heci_flush_list(&dev->ctrl_rd_list, conn, priv_cb);
	if (ret != 0) {
		spin_unlock_bh(&dev->crrdl_lock);
		return ret;
	}
	ret = heci_flush_list(&dev->ctrl_wr_list, conn, priv_cb);
	spin_unlock_bh(&dev->crrdl_lock);
	return ret;
}

/**
 * heci_disconnect_host_client - send disconnect message to fw from host client.
 *
 * @dev: Device object for our driver
 * @conn: connection data of the file object
 *
 * returns 0 on success, <0 on failure.
 */
int heci_disconnect_host_client(struct iamt_heci_device *dev,
			struct heci_connection *conn)
{
	int rets, err;
	long timeout = HECI_CONNECT_TIMEOUT; /* in seconds */
	struct heci_cb_private *priv_cb;

	if ((dev == NULL) || (conn == NULL))
		return -ENODEV;

	if (atomic_read(&conn->state) != HECI_FILE_DISCONNECTING)
		return 0;

	priv_cb = heci_init_io_cb(conn, HECI_CLOSE, NULL, 0, 0);
	if (priv_cb == NULL)
		return -ENOMEM;

	spin_lock_bh(&dev->cmdl_lock);
	list_add(&priv_cb->cb_list, &dev->heci_cmd_list.cb_list);
	spin_unlock_bh(&dev->cmdl_lock);
	atomic_inc(&dev->recvd_wrr);
	wake_up_interruptible(&dev->wait_isr);

	err = wait_event_timeout(conn->cc_wait,
		 (atomic_read(&conn->state) == HECI_FILE_DISCONNECTED),
		 timeout * HZ);
	if (atomic_read(&conn->state) == HECI_FILE_DISCONNECTED) {
		rets = 0;
		DBG("successfully disconnected from fw client.\n");
	} else {
		rets = -ENODEV;
		if (err)
			DBG("wait failed disconnect err=%08x\n", err);
		DBG("failed to disconnect from fw client.\n");
	}

	err = heci_flush_ctrl_lists(dev, conn, priv_cb);
	if (err != 0)
		heci_free_io_cb(priv_cb);

	return rets;
}

/**
 * heci_init_io_cb - allocates and initializes heci callback structure.
 *
 * @conn: connection data of the file object
 * @oper: callback operation type
 * @file_ext: private data of file object
 * @reqlen: length of request buffer
 * @rsplen: length of response buffer
 *
 * returns 0 on success, <0 on failure.
 */
struct heci_cb_private *heci_init_io_cb(struct heci_connection *conn,
					enum heci_cb_major_types oper,
					struct heci_file_priv *file_ext,
					size_t reqlen, size_t rsplen)
{
	struct heci_cb_private *priv_cb = NULL;

	priv_cb = kzalloc(sizeof(struct heci_cb_private), GFP_KERNEL);
	if (priv_cb == NULL)
		return NULL;

	INIT_LIST_HEAD(&priv_cb->cb_list);
	priv_cb->conn = conn;
	priv_cb->major_file_operations = oper;
	if (file_ext != NULL) {
		priv_cb->file_object = file_ext->file;
		priv_cb->file_ext = file_ext;
	}

	if (conn != NULL)
		atomic_inc(&conn->refcnt);

	if (reqlen > 0) {
		priv_cb->request_buffer.size = reqlen;
		priv_cb->request_buffer.data = kzalloc(reqlen, GFP_KERNEL);
		if (priv_cb->request_buffer.data == NULL) {
			kfree(priv_cb);
			return NULL;
		}
	}
	if (rsplen > 0) {
		priv_cb->response_buffer.size = rsplen;
		priv_cb->response_buffer.data = kzalloc(rsplen, GFP_KERNEL);
		if (priv_cb->response_buffer.data == NULL) {
			kfree(priv_cb->request_buffer.data);
			kfree(priv_cb);
			return NULL;
		}
	}
	atomic_set(&priv_cb->refcnt, 1);
	return priv_cb;
}

/**
 * heci_put_io_cb - free heci_cb_private related memory
 *
 * @priv_cb: heci callback struct
 *
 * returns 1 if free completed, 0 if not
 */
int heci_put_io_cb(struct heci_cb_private *priv_cb)
{
	if (priv_cb == NULL)
		return 1;

	if (!atomic_dec_and_test(&priv_cb->refcnt))
		return 0;

	kfree(priv_cb->request_buffer.data);
	kfree(priv_cb->response_buffer.data);
	if (priv_cb->conn != NULL)
		heci_put_heci_connection(priv_cb->conn);
	kfree(priv_cb);
	return 1;
}

/**
 * heci_get_io_cb - safely gets heci_cb_private related memory
 *
 * @priv_cb: heci callback struct
 *
 * returns heci callback struct
 */
struct heci_cb_private *heci_get_io_cb(struct heci_cb_private *priv_cb)
{
	if (priv_cb == NULL)
		return NULL;

	atomic_inc(&priv_cb->refcnt);
	return priv_cb;
}

